function install()
% install
%   Lets the user choose a path for libSBML and Mosek, and checks that
%   everything seems to be working. The paths to RAVEN Toolbox, libSBML and
%   Mosek are then stored in the Matlab path list  
%
%   Usage: install()
%
%   Rasmus Agren, 2013-03-05
%

%Get the directory of this file
[ST I]=dbstack('-completenames');
ravenDir=fileparts(ST(I).file);

%Choose the Mosek folder
mosekDir=uigetdir('','Select the Mosek home directory');
mosekDir=fullfile(mosekDir,'6','toolbox','r2009b');
if ~exist(fullfile(mosekDir,'mosekopt.m'),'file')
    throw(MException('','Not the correct MOSEK directory'));
end

%Choose the libSBML folder
sbmlDir=uigetdir('','Select the libSBML home directory');
sbmlDir=fullfile(sbmlDir,'bindings','matlab');
if ~exist(fullfile(sbmlDir,'TranslateSBML.m'),'file')
    if exist(fullfile(sbmlDir,'matlab','TranslateSBML.m'),'file')
        sbmlDir=fullfile(sbmlDir,'matlab');
    else
        throw(MException('','Not the correct libSBML directory'));
    end
end

%Add the folders to the PATH
addpath(ravenDir);
addpath(mosekDir);
addpath(sbmlDir);
savepath

%Test Mosek
a=optimset();
if isfield(a,'TolFun')
    throw(MException('','Wrong path list for Mosek'));
end

excelFile=fullfile(ravenDir,'tutorial','empty.xlsx');
xmlFile=fullfile(ravenDir,'tutorial','empty.xml');
try
    SBMLFromExcel(excelFile,xmlFile,false,false);
catch
    if ispc==1
        fprintf('ERROR: Could not import from Excel. Some functionality of the RAVEN Toolbox will not be available\n');
    else
        fprintf('ERROR: Could not import from Excel. This functionality uses Excel COM server, which works best for the Windows version of Matlab. Some functionality of the RAVEN Toolbox will not be available\n');
    end
end
try
    smallModel=importModel(xmlFile);
catch
    fprintf('ERROR: Could not use libSBML. Some functionality of the RAVEN Toolbox will not be available\n');
end
try
    sol=solveLP(smallModel);
catch
    fprintf('ERROR: Could not use Mosek. Some functionality of the RAVEN Toolbox will not be available\n');
end

%Everything is hunky dory
fprintf('INSTALLATION OF THE RAVEN TOOLBOX v1.02 COMPLETED\n');
end